==> locus-0.84.tar.gz: locus-0.84/bin/grazer/grazeopt.cpp <==
#include "myfndt.h"
#include "grazeopt.h"
#include "lversion.h"

#if !defined(PLATFORM_WIN32)
#include "txtcolor.h"
#endif
#include "tmpdir.h"

#include "omirror.h"
#include "ocolors.h"
#include "osoftop.h"
#include "ooutform.h"
#include "omode.h"
#include "oswild.h"
#include "ointrvl.h"
#include "otmp.h"
#include "ogrrpt.h"
#include "odisptxt.h"
#include "oquery.h"
#include "oqbexc.h"
#include "olang.h"
#include "olcase.h"
#include "oweb.h"
#include "oterm.h"

#include <stdio.h>
#if defined(PLATFORM_DOS)
#include <stdlib.h>
#endif

static double GetTamedWeight(COptions<double> *Options,
	const char *Key);
static int GetBoolFromInt(COptions<int> *Options, const char *Key);
static int GetSpotDist(COptions<int> *Options, const char *);

CGrazerOptions::CGrazerOptions(const DString &ExeDrive):
	CLocusOptions(ExeDrive)
{	
#if !defined(PLATFORM_WIN32)
	int Color;
#endif

	Add(O_WILDSPOTDIST, 8);
	Add(O_SPOTDIST, GetSpotDist);
#if !defined(PLATFORM_WIN32)
	Color = GetColorCode(TA_BOLD, TC_YELLOW, TC_BLACK);
	Add(O_DOCNAMECOLOR, Color);
	Color = GetColorCode(TA_NORMAL, TC_WHITE, TC_BLACK);
	Add(O_TEXTCOLOR, Color);
	Color = GetColorCode(TA_BOLD, TC_GREEN, TC_BLACK);
	Add(O_OCCCOLOR, Color);
	Color = GetColorCode(TA_BOLD, TC_YELLOW, TC_BLACK);
	Add(O_CLOTCOLOR, Color);
	Color = GetColorCode(TA_NORMAL, TC_WHITE, TC_BLACK);
	Add(O_DEFAULTCOLOR, Color);
#endif

	Add(O_DOCLIMIT, 0);
	double DefaultWeight = 0.2;
	Add(O_WILDFOCUSWEIGHT, DefaultWeight);
	Add(O_WILDCORRELATIONWEIGHT, DefaultWeight);
	Add(O_WILDPROMINENCEWEIGHT, DefaultWeight);
	Add(O_WILDLOCALITYWEIGHT, DefaultWeight);
	Add(O_WILDORDERWEIGHT, DefaultWeight);
	Add(O_FOCUSWEIGHT, GetTamedWeight);
	Add(O_CORRELATIONWEIGHT, GetTamedWeight);
	Add(O_PROMINENCEWEIGHT, GetTamedWeight);
	Add(O_LOCALITYWEIGHT, GetTamedWeight);
	Add(O_ORDERWEIGHT, GetTamedWeight);
	double RelevanceThreshold = 0.5;
	Add(O_RELEVANCETHRESHOLD, RelevanceThreshold);

	Add(O_SHOWRELWILD, 0);
	Add(O_SHOWREL, GetBoolFromInt);
	Add(O_HIGHLIGHTMAX, 0);
	Add(O_WORDLISTFILE, "");
	Add(O_REQWORDFILE, "");
	Add(O_PERVERSEWORDLIST, 0);

/*	Add(O_FORMATSTRING, DString(""));
#if !defined(PLATFORM_WIN32)
	Add(O_FORMATCOLOR,
		GetColorCode(TA_BOLD, TC_WHITE, TC_BLACK));
#endif*/
	Add(O_HEADLENGTH, 0);

	Add(O_INTERSTARTFILE, DString(""));
	Add(O_INTERSTARTOFFS, 0);
	Add(O_INCLUDEINTERSTART, 1);
	Add(O_INTERSTOPFILE, DString(""));
	Add(O_INTERSTOPOFFS, 0);
	Add(O_INCLUDEINTERSTOP, 1);
	Add(T_BDNOTFOUND_SDN, DString("Document %s from %d not found.\n"));
	Add(T_EMPTYINTERVAL_N, DString("Search interval is empty.\n"));

	Add(O_EXPLICITTEMPDIR, DString(""));
	Add(O_TEMPDIR, GetTempDir);
    Add(O_LATECASECONV, 2);

    Add(O_NOWORDS, "specify words to search for\n");
    Add(O_WARNHIDDEN, "warning: command-line args overriding stored query\n");
	Add(O_NOTFOUNDHEADER_0, "warning: the following words are not "
		   "indexed:\n");
	Add(O_NFLINE_1, "\t%0\n");
    Add(O_TIMEOUTSEC, (int)0);
    Add(O_SEARCHTIMEDOUT, "error: search timed out\n");

    Add(O_STARTINGSEARCH, "searching...\n");
	Add(O_SEARCHPROGRESS, "found %0 documents (in %1%% of the database)\n");

	Add(O_PROLOG_0, "");
	Add(O_EPILOG, "%0");
	Add(O_PASSAGEOVERFLOW_1, "The formatting of this document contains "
		   "a passage longer than %0 characters, which grazer does not handle. "
		   "Please change spot_distance option to view smaller portions of the "
             "document.\n");
	Add(O_WHOLEFILE_3, "%1File %0:\n");
	Add(O_PARTFILE_4, "%2File %0 (from %1)\n");
    Add(O_DOCEND, "");
	Add(O_METRIC_2, "%0 = %1\n");
	Add(O_RELEVANCE_1, "result = %0\n");
	Add(O_CLOTHEAD_1, "%0*");
	Add(O_CLOTBREAK_1, "%0\n...\n");
	Add(O_CLOTTAIL_1, "%0\n");
	Add(O_LOWPASS_4, "%3%0%1%2");
	Add(O_HIGHPASS_4, "%0%4%1%3%2");
    Add(O_TITLEPART, "");
    Add(O_CRUFT, "");
    Add(O_CRUFTCASE, 1);
	Add(O_OUTFILE, "");

	CStringListValue StringListValue;
	Add(O_SOFTOPS, StringListValue);
	Add(O_QEXPR, "");

	Add(T_QBMEMEXC_D, "exception: allocation of %d bytes failed");
	Add(T_QBOPENEXC_SS, "exception: could not open file %s with mode %s");
	Add(T_QBUNKNEXC, "exception: unknown exception");
	Add(T_PQWRAP_SD, "parsing query: %s on line %d");

	Add(T_UNKNOWNOP_S, "unknown operator %s");
	Add(T_DUPLWORD, "duplicate word as argument");
	Add(T_EMPTYQUERY, "operator without arguments");
    Add(O_UNKNOWNSUBDOC_1, "unknown document part %0");

    Add(O_DUPLWORD_1, "warning: word \"%0\" is duplicate\n");
    Add(O_SOUGHT, "searching for words:\n");
    Add(O_SOUGHTPRIMARY_1, "\t%0");
    Add(O_SOUGHTSECONDARY_1, " %0");
    Add(O_ENDVARIANT, "\n");

    Add(O_PARSEPARAM, 0);

	WatchSection("grazer");
}

CGrazerOptions::~CGrazerOptions()
{
}

DString CGrazerOptions::FirstInvalidKey()
{
	DString Key = CLocusOptions::FirstInvalidKey();
	if (!Key.IsEmpty())
		return Key;

     if (COptions<int>::Get(O_PERVERSEWORDLIST))
	{   DString RFile = Get(O_REQWORDFILE);
	    if (RFile.IsEmpty())
		   return DString(O_REQWORDFILE);
	}

	return DString("");
}

static double GetTamedWeight(COptions<double> *Options, 
	const char *Key)
{	static const char *OptKey[] = {O_WILDFOCUSWEIGHT,
		O_WILDCORRELATIONWEIGHT, O_WILDPROMINENCEWEIGHT,
		O_WILDLOCALITYWEIGHT, O_WILDORDERWEIGHT};

	double Sum = 0;
	for (int i = 0; i < sizeof(OptKey) / sizeof(OptKey[0]); i++)
	{	double Value = Options->Get(OptKey[i]);
		if (Value < 0)
			Value = 0;
		else
			if (Value > 1)
				Value = 1;
		Sum += Value;
	}
	if (!Sum)
		return 1 / (sizeof(OptKey) / sizeof(OptKey[0]));
	return Options->Get(Key + 1) / Sum;
}


static int GetBoolFromInt(COptions<int> *Options, const char *Key)
{
    Assert(Key != NULL && Key[0] =='_');
    return Options->Get(Key + 1) ? 1 : 0;
}

static int GetSpotDist(COptions<int> *Options, const char *)
{
    int SpotDist = Options->Get(O_WILDSPOTDIST);
    if (SpotDist < 2)
        return 2;

    if (SpotDist > 1000)
        return 1000;

    return SpotDist;
}

==> locus-0.88.tar.gz: locus-0.88/bin/grazer/grazeopt.cpp <==
#include "myfndt.h"
#include "grazeopt.h"
#include "lversion.h"

#if !defined(PLATFORM_WIN32)
#include "txtcolor.h"
#endif
#include "tmpdir.h"

#include "omirror.h"
#include "ocolors.h"
#include "osoftop.h"
#include "ooutform.h"
#include "omode.h"
#include "oswild.h"
#include "ointrvl.h"
#include "otmp.h"
#include "ogrrpt.h"
#include "odisptxt.h"
#include "oquery.h"
#include "oqbexc.h"
#include "olang.h"
#include "olcase.h"
#include "oweb.h"
#include "oterm.h"

#include <stdio.h>
#if defined(PLATFORM_DOS)
#include <stdlib.h>
#endif

static double GetTamedWeight(COptions<double> *Options,
	const char *Key);
static int GetBoolFromInt(COptions<int> *Options, const char *Key);
static int GetSpotDist(COptions<int> *Options, const char *);

CGrazerOptions::CGrazerOptions(const DString &ExeDrive):
	CLocusOptions(ExeDrive)
{	
#if !defined(PLATFORM_WIN32)
	int Color;
#endif

	Add(O_WILDSPOTDIST, 8);
	Add(O_SPOTDIST, GetSpotDist);
#if !defined(PLATFORM_WIN32)
	Color = GetColorCode(TA_BOLD, TC_YELLOW, TC_BLACK);
	Add(O_DOCNAMECOLOR, Color);
	Color = GetColorCode(TA_NORMAL, TC_WHITE, TC_BLACK);
	Add(O_TEXTCOLOR, Color);
	Color = GetColorCode(TA_BOLD, TC_GREEN, TC_BLACK);
	Add(O_OCCCOLOR, Color);
	Color = GetColorCode(TA_BOLD, TC_YELLOW, TC_BLACK);
	Add(O_CLOTCOLOR, Color);
	Color = GetColorCode(TA_NORMAL, TC_WHITE, TC_BLACK);
	Add(O_DEFAULTCOLOR, Color);
#endif

	Add(O_DOCLIMIT, 0);
	double DefaultWeight = 0.2;
	Add(O_WILDFOCUSWEIGHT, DefaultWeight);
	Add(O_WILDCORRELATIONWEIGHT, DefaultWeight);
	Add(O_WILDPROMINENCEWEIGHT, DefaultWeight);
	Add(O_WILDLOCALITYWEIGHT, DefaultWeight);
	Add(O_WILDORDERWEIGHT, DefaultWeight);
	Add(O_FOCUSWEIGHT, GetTamedWeight);
	Add(O_CORRELATIONWEIGHT, GetTamedWeight);
	Add(O_PROMINENCEWEIGHT, GetTamedWeight);
	Add(O_LOCALITYWEIGHT, GetTamedWeight);
	Add(O_ORDERWEIGHT, GetTamedWeight);
	double RelevanceThreshold = 0.5;
	Add(O_RELEVANCETHRESHOLD, RelevanceThreshold);

	Add(O_SHOWRELWILD, 0);
	Add(O_SHOWREL, GetBoolFromInt);
	Add(O_HIGHLIGHTMAX, 0);
	Add(O_WORDLISTFILE, "");
	Add(O_REQWORDFILE, "");
	Add(O_PERVERSEWORDLIST, 0);

/*	Add(O_FORMATSTRING, DString(""));
#if !defined(PLATFORM_WIN32)
	Add(O_FORMATCOLOR,
		GetColorCode(TA_BOLD, TC_WHITE, TC_BLACK));
#endif*/
	Add(O_HEADLENGTH, 0);

	Add(O_INTERSTARTFILE, DString(""));
	Add(O_INTERSTARTOFFS, 0);
	Add(O_INCLUDEINTERSTART, 1);
	Add(O_INTERSTOPFILE, DString(""));
	Add(O_INTERSTOPOFFS, 0);
	Add(O_INCLUDEINTERSTOP, 1);
	Add(T_BDNOTFOUND_SDN, DString("Document %s from %d not found.\n"));
	Add(T_EMPTYINTERVAL_N, DString("Search interval is empty.\n"));

	Add(O_EXPLICITTEMPDIR, DString(""));
	Add(O_TEMPDIR, GetTempDir);
    Add(O_LATECASECONV, 2);

    Add(O_NOWORDS, "specify words to search for\n");
    Add(O_WARNHIDDEN, "warning: command-line args overriding stored query\n");
	Add(O_NOTFOUNDHEADER_0, "warning: the following words are not "
		   "indexed:\n");
	Add(O_NFLINE_1, "\t%0\n");
    Add(O_TIMEOUTSEC, (int)0);
    Add(O_SEARCHTIMEDOUT, "error: search timed out\n");

    Add(O_STARTINGSEARCH, "searching...\n");
	Add(O_SEARCHPROGRESS, "found %0 documents (in %1%% of the database)\n");

	Add(O_PROLOG_0, "");
	Add(O_EPILOG, "%0");
	Add(O_PASSAGEOVERFLOW_1, "The formatting of this document contains "
		   "a passage longer than %0 characters, which grazer does not handle. "
		   "Please change spot_distance option to view smaller portions of the "
             "document.\n");
	Add(O_WHOLEFILE_X, "%1File %0:\n");
	Add(O_PARTFILE_X, "%2File %0 (from %1)\n");
    Add(O_DOCEND, "");
	Add(O_METRIC_2, "%0 = %1\n");
	Add(O_RELEVANCE_1, "result = %0\n");
	Add(O_CLOTHEAD_1, "%0*");
	Add(O_CLOTBREAK_1, "%0\n...\n");
	Add(O_CLOTTAIL_1, "%0\n");
	Add(O_LOWPASS_4, "%3%0%1%2");
	Add(O_HIGHPASS_4, "%0%4%1%3%2");
    Add(O_TITLEPARTS, CStringListValue());
	Add(O_OUTFILE, "");

	Add(O_SOFTOPS, CStringListValue());
	Add(O_QEXPR, "");

	Add(T_QBMEMEXC_D, "exception: allocation of %d bytes failed");
	Add(T_QBOPENEXC_SS, "exception: could not open file %s with mode %s");
	Add(T_QBUNKNEXC, "exception: unknown exception");
	Add(T_PQWRAP_SD, "parsing query: %s on line %d");

	Add(T_UNKNOWNOP_S, "unknown operator %s");
	Add(T_DUPLWORD, "duplicate word as argument");
	Add(T_EMPTYQUERY, "operator without arguments");
    Add(O_UNKNOWNSUBDOC_1, "unknown document part %0");

    Add(O_DUPLWORD_1, "warning: word \"%0\" is duplicate\n");
    Add(O_SOUGHT, "searching for words:\n");
    Add(O_SOUGHTPRIMARY_1, "\t%0");
    Add(O_SOUGHTSECONDARY_1, " %0");
    Add(O_ENDVARIANT, "\n");

    Add(O_PARSEPARAM, 0);

	WatchSection("grazer");
}

CGrazerOptions::~CGrazerOptions()
{
}

DString CGrazerOptions::FirstInvalidKey()
{
	DString Key = CLocusOptions::FirstInvalidKey();
	if (!Key.IsEmpty())
		return Key;

     if (COptions<int>::Get(O_PERVERSEWORDLIST))
	{   DString RFile = Get(O_REQWORDFILE);
	    if (RFile.IsEmpty())
		   return DString(O_REQWORDFILE);
	}

	return DString("");
}

static double GetTamedWeight(COptions<double> *Options, 
	const char *Key)
{	static const char *OptKey[] = {O_WILDFOCUSWEIGHT,
		O_WILDCORRELATIONWEIGHT, O_WILDPROMINENCEWEIGHT,
		O_WILDLOCALITYWEIGHT, O_WILDORDERWEIGHT};

	double Sum = 0;
	for (int i = 0; i < sizeof(OptKey) / sizeof(OptKey[0]); i++)
	{	double Value = Options->Get(OptKey[i]);
		if (Value < 0)
			Value = 0;
		else
			if (Value > 1)
				Value = 1;
		Sum += Value;
	}
	if (!Sum)
		return 1 / (sizeof(OptKey) / sizeof(OptKey[0]));
	return Options->Get(Key + 1) / Sum;
}


static int GetBoolFromInt(COptions<int> *Options, const char *Key)
{
    Assert(Key != NULL && Key[0] =='_');
    return Options->Get(Key + 1) ? 1 : 0;
}

static int GetSpotDist(COptions<int> *Options, const char *)
{
    int SpotDist = Options->Get(O_WILDSPOTDIST);
    if (SpotDist < 2)
        return 2;

    if (SpotDist > 1000)
        return 1000;

    return SpotDist;
}
